SystemOrganization addCategory: #'Seaside-RenderLoop'!

!WAPackage class methodsFor: '*seaside-renderloop' stamp: 'jok 11/28/2008 15:55'!
seasideRenderLoop
	^ self new
		name: 'Seaside-RenderLoop';
		description: 'Defines the low-level rendering and callback processing functionality.';
		addDependency: 'Seaside-Session';
		addDependency: 'Seaside-Component';
		yourself! !

!WASession methodsFor: '*seaside-renderloop' stamp: 'jf 2/20/2009 10:12'!
presenter
	^ self properties at: #presenter ifAbsent: [ nil ]! !

WASystemConfiguration subclass: #WARenderLoopConfiguration
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-RenderLoop'!

!WARenderLoopConfiguration commentStamp: 'jf 7/22/2008 12:32' prior: 0!
WARenderLoopConfiguration defines attributes required by the Render Loop classes (see the Seaside-Core-RenderLoop category). Any application using these classes (e.g. WARenderLoopMain) should include this configuration (this is done by default for new applications).

See #addAttributes: for details on provided attributes.!

!WARenderLoopConfiguration methodsFor: 'description' stamp: 'jf 10/17/2008 20:59'!
continuationClasses
	^ WARenderLoopContinuation allSubclasses asSortedCollection: [ :a :b | a name <= b name ]! !

!WARenderLoopConfiguration methodsFor: 'description' stamp: 'lr 1/2/2009 20:55'!
describeOn: config
	(config list: #actionPhaseContinuationClass)
		beAdvanced;
		group: #renderLoop;
		comment: 'The continuation used for the Render Loop''s action phase.';
		options: [ self continuationClasses ];
		default: WAActionPhaseContinuation.
	(config list: #renderPhaseContinuationClass)
		beAdvanced;
		group: #renderLoop;
		comment: 'The continuation used for the Render Loop''s render phase.';
		options: [ self continuationClasses ];
		default: WARenderPhaseContinuation.
	(config list: #rootClass)
		comment: 'The root component of this seaside application.';
		options: [ self rootClasses ].
	(config collection: #rootDecorationClasses)
		comment: 'The decoration clases that should wrap the root component.';
		options: [ self rootDecorationClasses ].
	config
		at: #mainClass put: WARenderLoopMain! !

!WARenderLoopConfiguration methodsFor: 'ancestry' stamp: 'jf 1/30/2009 17:40'!
parents
	"We override several attributes of the application configuration."
	^ Array with: WAApplicationConfiguration instance! !

!WARenderLoopConfiguration methodsFor: 'description' stamp: 'jf 7/29/2008 14:30'!
rootClasses
	^ (WAComponent allSubclasses select: [ :each | each canBeRoot ])
		asSortedCollection: [ :a :b | a name <= b name ]! !

!WARenderLoopConfiguration methodsFor: 'description' stamp: 'lr 9/14/2008 12:58'!
rootDecorationClasses
	^ (WADecoration allSubclasses select: [ :each | each canBeRoot ])
		asSortedCollection: [ :a :b | a name <= b name ]! !

WAError subclass: #WAMissingResponseError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-RenderLoop'!

!WAMissingResponseError methodsFor: 'accessing' stamp: 'jf 2/20/2009 18:09'!
seasidePossibleCauses
	^ #(
		'A Component attempted to use #call: or a similar method while rendering. Calling and answering can only be done in a callback.'
	)! !

WASessionContinuation subclass: #WARenderLoopContinuation
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-RenderLoop'!

WARenderLoopContinuation subclass: #WAActionPhaseContinuation
	instanceVariableNames: 'renderContext'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-RenderLoop'!

!WAActionPhaseContinuation methodsFor: 'processing' stamp: 'jf 2/8/2009 14:53'!
continue
	| renderContinuation |
	renderContinuation := self createRenderContinuation.
	renderContinuation captureState.
	(self shouldRedirect) 
		ifTrue: [ self redirectToContinuation: renderContinuation ]
		ifFalse: [ renderContinuation basicValue ]! !

!WAActionPhaseContinuation methodsFor: 'processing' stamp: 'jf 2/8/2009 14:53'!
handleRequest
	self runCallbacks.
	self continue! !

!WAActionPhaseContinuation methodsFor: 'accessing' stamp: 'jf 1/23/2009 12:05'!
renderContext
	^ renderContext! !

!WAActionPhaseContinuation methodsFor: 'accessing' stamp: 'jf 1/23/2009 12:06'!
renderContext: aRenderContext
	renderContext := aRenderContext! !

!WAActionPhaseContinuation methodsFor: 'processing' stamp: 'jf 2/8/2009 12:24'!
runCallbacks
	[ self withNotificationHandlerDo: [ 
		self renderContext callbacks handle: self requestContext ] ]
			ensure: [
				self request isXmlHttpRequest ifTrue: [ self captureState ] ]! !

!WAActionPhaseContinuation methodsFor: 'private' stamp: 'jf 10/19/2008 12:41'!
shouldRedirect
	| request |
	request := self request.
	^ (request isGet and: [ request fields includesKey: '_n' ]) not! !

!WARenderLoopContinuation methodsFor: 'private' stamp: 'jf 3/4/2009 09:48'!
createActionContinuation
	"Answer a new action phase continuation."

	^ (self application preferenceAt: #actionPhaseContinuationClass) new
			setStates: self states;
			yourself! !

!WARenderLoopContinuation methodsFor: 'private' stamp: 'jf 3/4/2009 09:49'!
createRenderContinuation
	"Answer a new render phase continuation."

	^ (self application preferenceAt: #renderPhaseContinuationClass) new! !

!WARenderLoopContinuation methodsFor: 'accessing' stamp: 'jf 2/20/2009 11:58'!
presenter
	"Answer the root presenter instance for the current session."

	^ self session presenter! !

!WARenderLoopContinuation methodsFor: 'updating' stamp: 'jf 2/20/2009 11:59'!
updateRoot: aHtmlRoot
	aHtmlRoot bodyAttributes at: 'onload' append: 'onLoad()' separator: ';'.
	super updateRoot: aHtmlRoot.
	(WAVisiblePresenterGuide client:
		(WAUpdateRootVisitor root: aHtmlRoot))
		start: self presenter! !

!WARenderLoopContinuation methodsFor: 'updating' stamp: 'jf 2/20/2009 11:59'!
updateStates: aSnapshot
	super updateStates: aSnapshot.
	(WAAllPresenterGuide client:
		(WAUpdateStatesVisitor snapshot: aSnapshot))
		start: self presenter! !

!WARenderLoopContinuation methodsFor: 'updating' stamp: 'jf 2/20/2009 11:59'!
updateUrl: anUrl
	super updateUrl: anUrl.
	(WAVisiblePresenterGuide client: (WAUpdateUrlVisitor url: anUrl))
		start: self presenter! !

!WARenderLoopContinuation methodsFor: 'private' stamp: 'jf 11/3/2008 08:25'!
withNotificationHandlerDo: aBlock
	^ aBlock on: WARenderNotification do: [ :n | ]! !

WARenderLoopContinuation subclass: #WARenderPhaseContinuation
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-RenderLoop'!

!WARenderPhaseContinuation methodsFor: 'private' stamp: 'jf 1/23/2009 12:13'!
createHtmlRootWithContext: aRenderContext
	| htmlRoot |
	htmlRoot := WAHtmlRoot context: aRenderContext.
	self updateRoot: htmlRoot.
	^ htmlRoot! !

!WARenderPhaseContinuation methodsFor: 'private' stamp: 'jf 1/23/2009 11:54'!
createRenderContext
	| renderContext |
	renderContext := WARenderContext new.
	
	"Wrap the default visitor with a rendering guide"
	renderContext visitor: (WARenderingGuide client: renderContext visitor).
	
	^ renderContext
		! !

!WARenderPhaseContinuation methodsFor: 'processing' stamp: 'jf 2/27/2009 21:06'!
handleRequest
	self session clearJumpTo.
	self respond: [ :response | self processRendering: response ].

	"The render phase should have returned a response"
	WAMissingResponseError signal! !

!WARenderPhaseContinuation methodsFor: 'processing' stamp: 'jf 2/21/2009 18:53'!
processRendering: aResponse 
	| document htmlRoot renderContext actionContinuation |
	"Prepare the response"
	aResponse contentType: self application contentType.

	"Create objects we need"
	document := self requestContext newDocument.
	actionContinuation := self createActionContinuation.
	renderContext := self createRenderContext.
	
	"Hook objects together"
	renderContext
		document: document;
		actionUrl: actionContinuation registerForUrl;
		resourceUrl: self application resourceBaseUrl.
	actionContinuation renderContext: renderContext.
	htmlRoot := self createHtmlRootWithContext: renderContext.

	"Render"
	document open: htmlRoot.
	self presenter renderWithContext: renderContext.
	document close.
	renderContext destroy! !

WAMain subclass: #WARenderLoopMain
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-RenderLoop'!

!WARenderLoopMain commentStamp: 'rew 10/19/2007 10:50' prior: 0!
When a new session on a WAApplication is started WARenderLoopMain initializes the application, that is it:
 
	creates the top level component of the application, 
	informs each component(WAPresenter) of the application that the session started (via WAPresenter>>initialRequest:)
	starts a WARenderLoop to handle the request
!

!WARenderLoopMain methodsFor: 'creational' stamp: 'lr 9/14/2008 13:14'!
createRoot
	^ self rootDecorationClasses
		inject: self rootClass new
		into: [ :component :decorationClass |
			component 
				addDecoration: decorationClass new;
				yourself ]! !

!WARenderLoopMain methodsFor: 'processing' stamp: 'jf 2/8/2009 14:54'!
prepareRoot: aComponent
	(WAAllPresenterGuide client:
		(WAInitialRequestVisitor request: self requestContext request))
		start: aComponent! !

!WARenderLoopMain methodsFor: 'accessing' stamp: 'lr 9/14/2008 12:56'!
rootClass
	^ self application preferenceAt: #rootClass! !

!WARenderLoopMain methodsFor: 'accessing' stamp: 'lr 9/14/2008 13:03'!
rootDecorationClasses
	^ self application preferenceAt: #rootDecorationClasses! !

!WARenderLoopMain methodsFor: 'processing' stamp: 'jf 3/4/2009 15:48'!
start
	| root |
	root := self createRoot.
	self session properties at: #presenter put: root.
	self prepareRoot: root.
	((self application preferenceAt: #renderPhaseContinuationClass) new) captureAndInvoke! !
